==> wiiuse_v0.1_src.tar.gz: wiiuse/src/wiiuse.c <==
/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2005
 *
 *	This file is part of wiiuse.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *	@brief General wiimote operations.
 *
 *	The file includes functions that handle general
 *	tasks.  Most of these are functions that are part
 *	of the API.
 */

#include <stdio.h>
#include <unistd.h>
#include <malloc.h>

#include "definitions.h"
#include "wiiuse_internal.h"
#include "events.h"

/**
 *	@brief Initialize an array of wiimote structures.
 *
 *	@param wiimotes		Number of wiimote_t structures to create.
 *	@param unids		An array of unique identifiers for each wiimote.
 *	@param event_cb		Callback to be executed on an event.
 *	@param stat_cb		Callback to be executed on an status event.
 *	@param dis_cb		Callback to be executed on a disconnection event.
 *
 *	@return An array of initialized wiimote_t structures.
 *
 *	@see wiimote_connect()
 *
 *	The array returned by this function can be passed to various
 *	functions, including wiimote_connect().
 */
struct wiimote_t** wiimote_init(int wiimotes, int* unids, wiimote_event_cb event_cb, wiimote_ctrl_status_cb stat_cb, wiimote_dis_cb dis_cb) {
	int i = 0;

	if (!wiimotes)
		return NULL;

	struct wiimote_t** wm = malloc(sizeof(struct wiimote_t*) * wiimotes);

	for (i = 0; i < wiimotes; ++i) {
		wm[i] = malloc(sizeof(struct wiimote_t));

		wm[i]->unid = unids[i];
		wm[i]->bdaddr = *BDADDR_ANY;
		wm[i]->out_sock = -1;
		wm[i]->in_sock = -1;
		wm[i]->leds = 0;
		wm[i]->state = WIIMOTE_INIT_STATES;

		wm[i]->event_cb = event_cb;
		wm[i]->dis_cb = dis_cb;
		wm[i]->stat_cb = stat_cb;

		wm[i]->read_req = NULL;

		wm[i]->handshake_state = 0;

		wm[i]->buttons_held = 0;
		wm[i]->buttons_released = 0;

		memset(wm[i]->event, 0, sizeof(wm[i]->event));
	}

	return wm;
}


/**
 *	@brief	The wiimote disconnected.
 *
 *	@param wm	Pointer to a wiimote_t structure.
 *
 *	Called when a wiimote disconnects.
 *	If a disconnect callback was registered, then invoke it.
 */
void wiimote_disconnected(struct wiimote_t* wm) {
	if (!wm)	return;

	DEBUG("Wiimote disconnected.");

	/* disable the connected flag */
	WIIMOTE_DISABLE_STATE(wm, WIIMOTE_STATE_CONNECTED);

	/* reset a bunch of stuff */
	wm->out_sock = -1;
	wm->in_sock = -1;
	wm->leds = 0;
	wm->state = WIIMOTE_INIT_STATES;
	wm->read_req = NULL;
	wm->handshake_state = 0;
	wm->buttons_held = 0;
	wm->buttons_released = 0;
	memset(wm->event, 0, sizeof(wm->event));

	/* call the disconnect callback if set */
	if (wm->dis_cb)
		wm->dis_cb(wm);
}


/**
 *	@brief	Enable or disable the rumble.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param status	1 to enable, 0 to disable.
 */
void wiimote_rumble(struct wiimote_t* wm, int status) {
	char buf[3];

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))	return;

	buf[0] = SET_REPORT | BT_OUTPUT;
	buf[1] = WIIMOTE_CMD_RUMBLE;

	/* make sure to keep the current lit leds */
	buf[2] = wm->leds;

	if (status) {
		DEBUG("Starting rumble...\n");
		WIIMOTE_ENABLE_STATE(wm, WIIMOTE_STATE_RUMBLE);
		buf[2] |= 0x01;
	} else {
		DEBUG("Stopping rumble...\n");
		WIIMOTE_DISABLE_STATE(wm, WIIMOTE_STATE_RUMBLE);
	}
	write(wm->out_sock, buf, 3);
}


/**
 *	@brief	Toggle the state of the rumble.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 */
void wiimote_toggle_rumble(struct wiimote_t* wm) {
	if (!wm)	return;

	wiimote_rumble(wm, !WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE));
}


/**
 *	@brief	Set the enabled LEDs.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param leds		What LEDs to enable.
 *
 *	\a leds is a bitwise or of WIIMOTE_LED_1, WIIMOTE_LED_2, WIIMOTE_LED_3, or WIIMOTE_LED_4.
 */
void wiimote_set_leds(struct wiimote_t* wm, int leds) {
	char buf[3];

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))	return;

	/* remove the lower 4 bits because they control rumble */
	wm->leds = (leds & 0xF0);

	/* make sure if the rumble is on that we keep it on */
	if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
		wm->leds |= 0x01;

	buf[0] = SET_REPORT | BT_OUTPUT;
	buf[1] = WIIMOTE_CMD_LED;
	buf[2] = wm->leds;

	write(wm->out_sock, buf, 3);
}


/**
 *	@brief	Set if the wiimote should report motion sensing.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param status	1 to enable, 0 to disable.
 *
 *	Since reporting motion sensing sends a lot of data,
 *	the wiimote saves power by not transmitting it
 *	by default.
 */
void wiimote_motion_sensing(struct wiimote_t* wm, int status) {
	char buf[4];

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))	return;

	buf[0] = SET_REPORT | BT_OUTPUT;
	buf[1] = WIIMOTE_CMD_REPORT_TYPE;
	buf[2] = 0x00;			/* set to 0x04 for continuous reporting */

	/* if rumble is enabled, make sure we keep it */
	if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
		buf[2] |= 0x01;

	if (status) {
		/* enable */
		DEBUG("Starting motion sensing...\n");
		buf[3] = 0x31;
	} else {
		/* disable */
		DEBUG("Stopping motion sensing...\n");
		buf[3] = 0x30;
	}

	write(wm->out_sock, buf, 4);
}


/**
 *	@brief	Read data from the wiimote.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param read_cb	Function pointer to call when the data arrives from the wiimote.
 *	@param buffer	An allocated buffer to store the data as it arrives from the wiimote.
 *					Must be persistent in memory and large enough to hold the data.
 *	@param offset	The offset into the wiimote memory to start reading from.
 *	@param len		The length of the block to be read.
 *
 *	The library can only handle one data read request at a time
 *	because it must keep track of the buffer and other
 *	events that are specific to that request.  So if a request
 *	has already been made, subsequent requests will be added
 *	to a pending list and be sent out when the previous
 *	finishes.
 */
int wiimote_read_data(struct wiimote_t* wm, wiimote_read_cb read_cb, byte* buffer, unsigned int offset, unsigned short len) {
	if (!wm)		return 0;
	if (!WIIMOTE_IS_CONNECTED(wm))	return 0;
	if (!buffer)	return 0;
	if (!len)		return 0;
	if (!read_cb)	return 0;

	#if 0
	if (wm->read_wait) {
		/* still waiting for data from the last read */
		return 0;
	}

	wm->read_cb = read_cb;
	wm->read_buf = buffer;
	wm->read_offset = offset;
	wm->read_size = len;
	wm->read_wait = len;
	#endif

	/* make this request structure */
	struct read_req_t* req = (struct read_req_t*)malloc(sizeof(struct read_req_t));
	req->cb = read_cb;
	req->buf = buffer;
	req->offset = offset;
	req->size = len;
	req->wait = len;
	req->next = NULL;

	/* add this to the request list */
	if (!wm->read_req) {
		/* root node */
		wm->read_req = req;

		/* send the request out immediately */
		wiimote_send_next_pending_read_request(wm);
	} else {
		struct read_req_t* nptr = wm->read_req;
		for (; nptr->next; nptr = nptr->next);
		nptr->next = req;

		DEBUG("Added pending data read request.");
	}

	return 1;
}


/**
 *	@brief Send the next pending data read request to the wiimote.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	@see wiimote_read_data()
 *
 *	This function is not part of the wiiuse API.
 */
void wiimote_send_next_pending_read_request(struct wiimote_t* wm) {
	byte buf[8];

	if (!wm)			return;
	if (!wm->read_req)	return;

	struct read_req_t* req = wm->read_req;

	buf[0] = SET_REPORT | BT_OUTPUT;
	buf[1] = WIIMOTE_CMD_READ_DATA;

	/* the offset is in big endian */
	*(int*)(buf + 2) = BIG_ENDIAN_LONG(req->offset);

	/* first low bit of the first byte of the address is the rumble */
	if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
		buf[2] |= 0x01;

	/* the length is in big endian */
	*(short*)(buf + 6) = BIG_ENDIAN_SHORT(req->size);

	DEBUG("(%x) %x %x %x %x %x %x %x\n", buf[0],  buf[1],  buf[2],  buf[3],  buf[4],  buf[5], buf[6], buf[7]);

	write(wm->out_sock, buf, 8);
}


/**
 *	@brief Request the wiimote controller status.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	Controller status includes: battery level, LED status, extensions
 */
void wiimote_status(struct wiimote_t* wm) {
	char buf[3];

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))	return;

	buf[0] = SET_REPORT | BT_OUTPUT;
	buf[1] = WIIMOTE_CMD_CTRL_STATUS;
	buf[2] = 0x00;

	#if 0
	/* if rumble is enabled, make sure we keep it */
	if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
		buf[2] |= 0x01;
	#endif

	DEBUG("Requested wiimote status.");

	write(wm->out_sock, buf, 3);
}


/**
 *	@brief Send a report type packet to the wiimote.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	This function is not part of the wiiuse API.
 */
void wiimote_send_report_type(struct wiimote_t* wm) {
	/* the same packet type is send when setting the motion sensing */
	wiimote_motion_sensing(wm, WIIMOTE_IS_SET(wm, WIIMOTE_STATE_MOTION_SENSING));
}


/**
 *	@brief Find a wiimote_t structure by its unique identifier.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param wiimotes	The number of wiimote_t structures in \a wm.
 *	@param unid		The unique identifier to search for.
 *
 *	@return Pointer to a wiimote_t structure, or NULL if not found.
 */
struct wiimote_t* wiimote_get_by_id(struct wiimote_t** wm, int wiimotes, int unid) {
	int i = 0;

	for (; i < wiimotes; ++i) {
		if (wm[i]->unid == unid)
			return wm[i];
	}

	return NULL;
}

==> wiiuse_v0.2_src.tar.gz: wiiuse/src/wiiuse.c <==
/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2005
 *
 *	This file is part of wiiuse.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *	@brief General wiimote operations.
 *
 *	The file includes functions that handle general
 *	tasks.  Most of these are functions that are part
 *	of the API.
 */

#include <stdio.h>
#include <unistd.h>
#include <malloc.h>

#include "definitions.h"
#include "wiiuse_internal.h"
#include "events.h"

/*
 *	Error code strings.
 */
static struct errno_str_t errno_str[] = {
	{	ERR_NONE,				"No error."							},
	{	ERR_NOT_CONNECTED,		"The wiimote is not connected."		},
	{	ERR_CREATING_SOCKET,	"Unable to create socket."			},
	{	ERR_CONNECT_SOCKET,		"Unable to connect to socket."		},
};


/**
 *	@brief Initialize an array of wiimote structures.
 *
 *	@param wiimotes		Number of wiimote_t structures to create.
 *	@param unids		An array of unique identifiers for each wiimote.
 *	@param event_cb		Callback to be executed on an event.
 *	@param stat_cb		Callback to be executed on an status event.
 *	@param dis_cb		Callback to be executed on a disconnection event.
 *
 *	@return An array of initialized wiimote_t structures.
 *
 *	@see wiimote_connect()
 *
 *	The array returned by this function can be passed to various
 *	functions, including wiimote_connect().
 */
struct wiimote_t** wiimote_init(int wiimotes, int* unids, wiimote_event_cb event_cb, wiimote_ctrl_status_cb stat_cb, wiimote_dis_cb dis_cb) {
	int i = 0;

	if (!wiimotes)
		return NULL;

	struct wiimote_t** wm = malloc(sizeof(struct wiimote_t*) * wiimotes);

	for (i = 0; i < wiimotes; ++i) {
		wm[i] = malloc(sizeof(struct wiimote_t));
		memset(wm[i], 0, sizeof(struct wiimote_t));

		wm[i]->unid = unids[i];
		wm[i]->bdaddr = *BDADDR_ANY;
		wm[i]->out_sock = -1;
		wm[i]->in_sock = -1;
		wm[i]->state = WIIMOTE_INIT_STATES;

		wm[i]->event_cb = event_cb;
		wm[i]->dis_cb = dis_cb;
		wm[i]->stat_cb = stat_cb;

		wm[i]->expansion.type = EXP_NONE;
	}

	return wm;
}


/**
 *	@brief	The wiimote disconnected.
 *
 *	@param wm	Pointer to a wiimote_t structure.
 *
 *	Called when a wiimote disconnects.
 *	If a disconnect callback was registered, then invoke it.
 */
void wiimote_disconnected(struct wiimote_t* wm) {
	if (!wm)	return;

	DEBUG("Wiimote disconnected.");

	/* disable the connected flag */
	WIIMOTE_DISABLE_STATE(wm, WIIMOTE_STATE_CONNECTED);

	/* reset a bunch of stuff */
	wm->out_sock = -1;
	wm->in_sock = -1;
	wm->leds = 0;
	wm->state = WIIMOTE_INIT_STATES;
	wm->read_req = NULL;
	wm->handshake_state = 0;
	wm->btns = 0;
	wm->btns_held = 0;
	wm->btns_released = 0;
	memset(wm->event, 0, sizeof(wm->event));

	/* call the disconnect callback if set */
	if (wm->dis_cb)
		wm->dis_cb(wm);
}


/**
 *	@brief	Enable or disable the rumble.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param status	1 to enable, 0 to disable.
 */
void wiimote_rumble(struct wiimote_t* wm, int status) {
	byte buf;

	if (!wm) return;
	if (!WIIMOTE_IS_CONNECTED(wm))
		RETN_ERRNO(wm, ERR_NOT_CONNECTED);

	/* make sure to keep the current lit leds */
	buf = wm->leds;

	if (status) {
		DEBUG("Starting rumble...\n");
		WIIMOTE_ENABLE_STATE(wm, WIIMOTE_STATE_RUMBLE);
		buf |= 0x01;
	} else {
		DEBUG("Stopping rumble...\n");
		WIIMOTE_DISABLE_STATE(wm, WIIMOTE_STATE_RUMBLE);
	}

	wiimote_send(wm, WIIMOTE_CMD_RUMBLE, &buf, 1);
}


/**
 *	@brief	Toggle the state of the rumble.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 */
void wiimote_toggle_rumble(struct wiimote_t* wm) {
	if (!wm)	return;

	wiimote_rumble(wm, !WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE));
}


/**
 *	@brief	Set the enabled LEDs.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param leds		What LEDs to enable.
 *
 *	\a leds is a bitwise or of WIIMOTE_LED_1, WIIMOTE_LED_2, WIIMOTE_LED_3, or WIIMOTE_LED_4.
 */
void wiimote_set_leds(struct wiimote_t* wm, int leds) {
	byte buf;

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))
		RETN_ERRNO(wm, ERR_NOT_CONNECTED);

	/* remove the lower 4 bits because they control rumble */
	wm->leds = (leds & 0xF0);

	/* make sure if the rumble is on that we keep it on */
	if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
		wm->leds |= 0x01;

	buf = wm->leds;

	wiimote_send(wm, WIIMOTE_CMD_LED, &buf, 1);
}


/**
 *	@brief	Set if the wiimote should report motion sensing.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param status	1 to enable, 0 to disable.
 *
 *	Since reporting motion sensing sends a lot of data,
 *	the wiimote saves power by not transmitting it
 *	by default.
 */
void wiimote_motion_sensing(struct wiimote_t* wm, int status) {
	if (status)
		WIIMOTE_ENABLE_STATE(wm, WIIMOTE_STATE_MOTION_SENSING);
	else
		WIIMOTE_DISABLE_STATE(wm, WIIMOTE_STATE_MOTION_SENSING);

	wiimote_set_report_type(wm);
}


/**
 *	@brief	Set the report type based on the current wiimote state.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	This is needed if motion sensing, expansion port, or IR has
 *	been toggled.
 */
void wiimote_set_report_type(struct wiimote_t* wm) {
	byte buf[2];
	int motion, exp;

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))
		RETN_ERRNO(wm, ERR_NOT_CONNECTED);

	buf[0] = 0x00;			/* set to 0x04 for continuous reporting */
	buf[1] = 0x00;

	/* if rumble is enabled, make sure we keep it */
	if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
		buf[0] |= 0x01;

	motion = WIIMOTE_IS_SET(wm, WIIMOTE_STATE_MOTION_SENSING);
	exp = WIIMOTE_IS_SET(wm, WIIMOTE_STATE_USE_EXPANSION);

	if (motion && exp) {
		/* motion sensing & expansion */
		DEBUG("Setting report type: motion sensing, expansion...\n");
		buf[1] = 0x37;
	} else if (exp) {
		/* expansion */
		DEBUG("Setting report type: expansion...\n");
		buf[1] = 0x36;
	} else if (motion) {
		/* motion sensing */
		DEBUG("Setting report type: motion sensing...\n");
		buf[1] = 0x31;
	} else {
		/* buttons only */
		DEBUG("Setting report type: buttons...\n");
		buf[1] = 0x30;
	}

	wiimote_send(wm, WIIMOTE_CMD_REPORT_TYPE, buf, 2);
}


/**
 *	@brief	Read data from the wiimote.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param read_cb	Function pointer to call when the data arrives from the wiimote.
 *	@param buffer	An allocated buffer to store the data as it arrives from the wiimote.
 *					Must be persistent in memory and large enough to hold the data.
 *	@param addr		The address of wiimote memory to read from.
 *	@param len		The length of the block to be read.
 *
 *	The library can only handle one data read request at a time
 *	because it must keep track of the buffer and other
 *	events that are specific to that request.  So if a request
 *	has already been made, subsequent requests will be added
 *	to a pending list and be sent out when the previous
 *	finishes.
 */
int wiimote_read_data(struct wiimote_t* wm, wiimote_read_cb read_cb, byte* buffer, unsigned int addr, unsigned short len) {
	if (!wm)		return 0;
	if (!WIIMOTE_IS_CONNECTED(wm))
		RET_ERRNO(wm, ERR_NOT_CONNECTED, 0);
	if (!buffer || !len || !read_cb)
		return 0;

	/* make this request structure */
	struct read_req_t* req = (struct read_req_t*)malloc(sizeof(struct read_req_t));
	req->cb = read_cb;
	req->buf = buffer;
	req->addr = addr;
	req->size = len;
	req->wait = len;
	req->next = NULL;

	/* add this to the request list */
	if (!wm->read_req) {
		/* root node */
		wm->read_req = req;

		/* send the request out immediately */
		wiimote_send_next_pending_read_request(wm);
	} else {
		struct read_req_t* nptr = wm->read_req;
		for (; nptr->next; nptr = nptr->next);
		nptr->next = req;

		DEBUG("Added pending data read request.");
	}

	return 1;
}


/**
 *	@brief Send the next pending data read request to the wiimote.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	@see wiimote_read_data()
 *
 *	This function is not part of the wiiuse API.
 */
void wiimote_send_next_pending_read_request(struct wiimote_t* wm) {
	byte buf[6];

	if (!wm)			return;
	if (!wm->read_req)	return;

	struct read_req_t* req = wm->read_req;

	/* the offset is in big endian */
	*(int*)(buf) = BIG_ENDIAN_LONG(req->addr);

	/* the length is in big endian */
	*(short*)(buf + 4) = BIG_ENDIAN_SHORT(req->size);

	DEBUG("Request read at address: 0x%x  length: %i", req->addr, req->size);
	wiimote_send(wm, WIIMOTE_CMD_READ_DATA, buf, 6);
}


/**
 *	@brief Request the wiimote controller status.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	Controller status includes: battery level, LED status, extensions
 */
void wiimote_status(struct wiimote_t* wm) {
	byte buf = 0;

	if (!wm)	return;
	if (!WIIMOTE_IS_CONNECTED(wm))
		RETN_ERRNO(wm, ERR_NOT_CONNECTED);

	DEBUG("Requested wiimote status.");

	wiimote_send(wm, WIIMOTE_CMD_CTRL_STATUS, &buf, 1);
}


/**
 *	@brief Send a report type packet to the wiimote.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	This function is not part of the wiiuse API.
 */
void wiimote_send_report_type(struct wiimote_t* wm) {
	/* the same packet type is send when setting the motion sensing */
	wiimote_motion_sensing(wm, WIIMOTE_IS_SET(wm, WIIMOTE_STATE_MOTION_SENSING));
}


/**
 *	@brief Find a wiimote_t structure by its unique identifier.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param wiimotes	The number of wiimote_t structures in \a wm.
 *	@param unid		The unique identifier to search for.
 *
 *	@return Pointer to a wiimote_t structure, or NULL if not found.
 */
struct wiimote_t* wiimote_get_by_id(struct wiimote_t** wm, int wiimotes, int unid) {
	int i = 0;

	for (; i < wiimotes; ++i) {
		if (wm[i]->unid == unid)
			return wm[i];
	}

	return NULL;
}


/**
 *	@brief	Write data to the wiimote.
 *
 *	@param wm			Pointer to a wiimote_t structure.
 *	@param addr			The address to write to.
 *	@param data			The data to be written to the memory location.
 *	@param len			The length of the block to be written.
 */
int wiimote_write_data(struct wiimote_t* wm, unsigned int addr, byte* data, byte len) {
	/* TODO - is write_loc actually needed? if not, flags in wiimote_send() is not needed either */
	byte buf[21] = {0};		/* the payload is always 23 */

	if (!wm)		return 0;
	if (!WIIMOTE_IS_CONNECTED(wm))
		RET_ERRNO(wm, ERR_NOT_CONNECTED, 0);
	if (!data || !len)
		return 0;

	DEBUG("Writing %i bytes to memory location 0x%x...", len, addr);

	#ifdef _DEBUG_
	{
		int i = 0;
		printf("Write data is: ");
		for (; i < len; ++i)
			printf("%x ", data[i]);
		printf("\n");
		getchar();
	}
	#endif

	/* the offset is in big endian */
	*(int*)(buf) = BIG_ENDIAN_LONG(addr);

	/* length */
	*(byte*)(buf + 4) = len;

	/* data */
	memcpy(buf + 5, data, len);

	wiimote_send(wm, WIIMOTE_CMD_WRITE_DATA, buf, 21);
	return 1;
}


/**
 *	@brief	Send a packet to the wiimote.
 *
 *	@param wm			Pointer to a wiimote_t structure.
 *	@param report_type	The report type to send (WIIMOTE_CMD_LED, WIIMOTE_CMD_RUMBLE, etc). Found in wiiuse.h
 *	@param msg			The payload.
 *	@param len			Length of the payload in bytes.
 *
 *	This function should replace any write()s directly to the wiimote device.
 */
void wiimote_send(struct wiimote_t* wm, int report_type, byte* msg, int len) {
	byte buf[32];		/* no payload is better than this */
	int rumble = 0;

	buf[0] = SET_REPORT | BT_OUTPUT;
	buf[1] = report_type;

	switch (report_type) {
		case WIIMOTE_CMD_LED:
		case WIIMOTE_CMD_RUMBLE:
		case WIIMOTE_CMD_CTRL_STATUS:
		{
			/* Rumble flag for: 0x11, 0x13, 0x14, 0x15, 0x19 or 0x1a */
			if (WIIMOTE_IS_SET(wm, WIIMOTE_STATE_RUMBLE))
				rumble = 1;
			break;
		}
		default:
			break;
	}

	memcpy(buf+2, msg, len);

	if (rumble)
		buf[2] |= 0x01;

	#ifdef _DEBUG_
	{
		int x = 2;
		printf("[DEBUG] SEND: (%x) %x ", buf[0], buf[1]);
		for (; x < len+2; ++x)
			printf("%.2x ", buf[x]);
		printf("\n");
	}
	#endif

	write(wm->out_sock, buf, len+2);
}


/**
 *	@brief Return the string representation of the last error code.
 *
 *	@param wm			Pointer to a wiimote_t structure.
 */
const char* wiimote_get_errno_str(struct wiimote_t* wm) {
	if (wm->errno > (sizeof(errno_str) / sizeof(struct errno_str_t)))
		return NULL;

	return errno_str[wm->errno].str;
}
